.DBcon <- function(){
  source("~/database.conf.R")
  for(var in c('.db.driver','.db.user','.db.pass','.db.name','.db.host',
               '.db.stats.tab.nm','.db.cruise.fkey.nm','.db.cruise.tab.nm','.db.sds.tab.nm'))
  assign(var, get(var), envir=.GlobalEnv)
  
  require(RPostgreSQL)

  dbConnect(dbDriver(.db.driver),
            user=.db.user, 
            password=.db.pass, 
            dbname=.db.name, 
            host=.db.host)
  
}

.loadStats <- function(cruise, first.time = FALSE, update=FALSE, con=.DBcon()){
  ## assumes you've already constructed a database table. with proper column names and types
  ## if not, please run the SQL script to create this table

  ## find the cruise id fromt the cruise table
  cruises <- dbReadTable(con, .db.cruise.tab.nm)
  cruise.id <- cruises$id[cruises$name_dir==cruise]

  ct <- dbGetQuery(con, paste("SELECT count(*) FROM", .db.stats.tab.nm, "WHERE", .db.cruise.fkey.nm, "=",cruise.id))[[1]]

  if(ct > 0){
      if(update==FALSE){
        stop(paste("Found",ct,"prexisting statistic records for this cruise. Try setting update to TRUE"))
      }else{
        dum <- dbGetQuery(con, paste("DELETE FROM", .db.stats.tab.nm, "WHERE", .db.cruise.fkey.nm, "=",cruise.id))[[1]]
        print(paste("deleted", ct, "statistics records from the database"))
      }
  }else{
    print('Fresh cruise stats for the database!')
  }

  ## read in the actual table to import
  stats.tab <- read.delim(paste(REPO.PATH,cruise ,'/stats.tab',sep=''))

  ## add the cruise id
  stats.tab.names <- names(stats.tab)
  stats.tab <- cbind.data.frame(stats.tab, as.integer(cruise.id))
  names(stats.tab) <- c(stats.tab.names, .db.cruise.fkey.nm)

  ## add the primary key
  if(!first.time){
    n <- dbGetQuery(con, paste("SELECT id FROM",.db.stats.tab.nm,"ORDER BY id DESC LIMIT 1"))[[1]]
    stats.tab$id <- 1:nrow(stats.tab) + n
  }
  #add special db-only outlier identifier column
  stats.tab$outlier <- FALSE

  ## reorder the columns of our dataframe before loading
  db.stat.cols <- dbListFields(con, .db.stats.tab.nm)
  reordered.names <- names(stats.tab)[match(db.stat.cols, names(stats.tab))]
  if(any(is.na(reordered.names)))
    stop('Too many unmatched columns to database column list. Stopping')
  stats.tab <- stats.tab[ ,reordered.names]

  print("loading statistics table to database")
  dbWriteTable(con, .db.stats.tab.nm, stats.tab, row.names=FALSE, append=TRUE)
  
}	



.loadSDS <- function(cruise, first.time = FALSE, con=.DBcon()){
  ## assumes you've already constructed a database table. with proper column names and types
  ## if not, please run the SQL script to create this table

  ## find the cruise id fromt the cruise table
  cruises <- dbReadTable(con, .db.cruise.tab.nm)
  cruise.id <- cruises$id[cruises$name_dir==cruise]

  ct <- dbGetQuery(con, paste("SELECT count(*) FROM",.db.sds.tab.nm,"WHERE", .db.cruise.fkey.nm, "=",cruise.id))[[1]]

  if(ct > 0){
    warning(paste("Found",ct,"prexisting sds records for this cruise, skipping sds load"))
  }else{
  
    ## read in the actual table to import 
    SDSs <- read.delim(paste(REPO.PATH,cruise ,'/sds.tab',sep=''))
 
  
    ## add the cruise id 
    sds.names <- names(SDSs)
    SDSs <- cbind.data.frame(SDSs, rep(as.integer(cruise.id),nrow(SDSs)))
    names(SDSs) <- c(sds.names, .db.cruise.fkey.nm)
    ## remove the null columns
    SDSs <- SDSs[,!(1:ncol(SDSs) %in% grep( 'NULL', names(SDSs)))] 


    ## add the primary key
    if(!first.time){
      n <- dbGetQuery(con, paste("SELECT id FROM",.db.sds.tab.nm,"ORDER BY id DESC LIMIT 1"))[[1]]
      SDSs$id <- 1:nrow(SDSs) + n
    }

    ## reorder the columns of our dataframe before loading
    db.file.cols <- dbListFields(con,.db.sds.tab.nm)
    reordered.names <- names(SDSs)[match(db.file.cols, gsub('\\.','_',tolower(names(SDSs))))]
    if(any(is.na(reordered.names)))
      stop('Too many unmatched columns to database column list. Stopping')
    SDSs <- SDSs[ ,reordered.names]

    print("loading sds table to database")
    dbWriteTable(con, .db.sds.tab.nm, SDSs, row.names=FALSE, append=TRUE)
    }
}	




.quotize <- function(var.vect)
  sapply(var.vect, function(x) paste("'",x,"'",sep=''))

  

.queryStats <- function(cruise, x.var='map',
                       y.vars = c('conc','fsc','chl'),
                       ranges = list(utc=c('2009-01-01','2030-01-01'),lat=c(-90,90),long=c(-180,180)),
                       populationsp = c('pico','ultra','nano','synecho','crypto')){
  if(FALSE){
    cruise <- 'Thompson'
    x.var <- 'time'
    y.vars <- c('conc','fsc','chl')
    ranges <- list(utc=c('2009-11-07','2009-11-09'),lat=c(-90,90),long=c(-180,180)) # .00,24.
    populations = c('pico','ultra','nano','synecho','crypto')
  }

  what <- paste("pop,", paste(y.vars,collapse=','))
  if(x.var == 'map'){
    what <- paste(what, ",lat,long")
  }else{
    what <- paste(what, " ,'", x.var,"'", sep='')
  }

  where <- paste(.db.stats.tab.nm,'.',.db.cruise.fkey.nm, '=',.db.cruise.tab.nm,".id AND name ='", cruise,"'", sep='')
  where <- paste(where, 'AND pop IN (', paste(.quotize(populations), collapse=','),')')
  for(range.name in names(ranges)){
    range <- ranges[range.name][[1]]
    if(all(sapply(range, is.character)))   #range.name == 'utc'
      range <- .quotize(range)
    where <- paste(where, 'AND', range.name, 'BETWEEN (', paste(range, collapse=' AND '),')')
  }

  
  sql <- paste("SELECT", what ,"FROM", .db.stats.tab.nm,',',.db.cruise.tab.nm, "WHERE", where,';')
  #rs <- dbSendQuery(con, sql)
  #df <- fetch(rs)
  #return(df)
  return(sql)

}

